#! /usr/bin/env python
#
# Example program using irc.bot.
#
# Joel Rosdahl <joel@rosdahl.net>

"""A simple example bot.
This is an example bot that uses the SingleServerIRCBot class from
irc.bot.  The bot enters a channel and listens for commands in
private messages and channel traffic.  Commands in channel messages
are given by prefixing the text by the bot name followed by a colon.
It also responds to DCC CHAT invitations and echos data sent in such
sessions.
The known commands are:
    stats -- Prints some channel information.
    disconnect -- Disconnect the bot.  The bot will try to reconnect
                  after 60 seconds.
    die -- Let the bot cease to exist.
    dcc -- Let the bot invite you to a DCC CHAT connection.
"""

import irc.bot
import irc.strings
from irc.client import ip_numstr_to_quad, ip_quad_to_numstr
import random, string, subprocess
from makeMexicanWall import saveMerica

def randomWord(length):
	return "".join(random.choice(string.ascii_lowercase) for i in range(length))


#setup ii
class IIInstance():
	def __init__(self, server, nick, port=6667, path="./irc/"):
		args = ['/usr/bin/ii', '-s',server, '-n', nick, '-p',str(port), '-i', path]
		self.h_ii = subprocess.Popen(args)
		print("Connected to " + server + " as "  + nick + " via ii")
		self.path = path + server + "/"

	def isConnected(self):
		return self.h_ii.poll() != ""

	def joinChannel(self, channel):
		if self.isConnected():
			fin = open(self.path + "in", 'w')
			fin.write("/j " + channel + "\n")
			fin.close()

	def query(self, user, msg):
		if self.isConnected():
			fin = open(self.path + "in" , 'w')
			fin.write("/j " + user + " " + msg)
			fin.close()

	def leaveChannel(self, channel):
		if self.isConnected():
			fin = open(self.path + "in", 'w')
			fin.write("/l " + channel + "\n")
			fin.close()
	
	def getLog(self, channel):
		return self.path + "#" + channel[1:] + "/out"

	def stop(self):
		self.h_ii.terminate()


class TestBot(irc.bot.SingleServerIRCBot):
	def __init__(self, channel, nickname, server, port=6667):
		#initilize empty user list
		self.known_users = {}	

		#Connect to irc-server 
		irc.bot.SingleServerIRCBot.__init__(self, [(server, port)], nickname, nickname)
		self.channel = channel
		print("Connecting to " + channel + "@" + server + " as " + nickname)
	
		self.pii = IIInstance(server, randomWord(6)+"bot", port)

	def on_nicknameinuse(self, c, e):
		c.nick(c.get_nickname() + "_")

	def on_welcome(self, c, e):
		c.join(self.channel)

	def on_privmsg(self, c, e):
		#get user informations
		nick = e.source.nick
		msg = e.arguments[0]

		if nick not in self.known_users:
	
			# Greet the user in the right manner
			c.notice(nick, "I feel you float around me,")
			c.notice(nick, "Spirit, I summon to appear, speak to me!") 
			c.notice(nick, "Ah! What tears now at the core of me!" )
			c.notice(nick, "All my sensees reeling") 
			c.notice(nick, "With fresh feeling!" )
			c.notice(nick, "I feel you draw my whole heart towards you!")
			c.notice(nick, "You must! You must! Though my Life`s lost, too!")
			self.known_users[nick] = "pending"

		elif self.known_users[nick] is "pending":
			if msg == "Who calls me?":
				c.notice(nick, "You shall pass!")
				
				#Connect to channel and wait
				channel = "#" + randomWord(16)
				self.pii.joinChannel(channel)
				c.notice(nick, "Find me at " + channel + " and you shall get your fish")
				self.known_users[nick] = channel
			else:
				c.notice(nick, "Terrible to gaze at!")

		else :
			self.do_command(nick, msg)

	def on_dccmsg(self, c, e):
	# non-chat DCC messages are raw bytes; decode as text
		text = e.arguments[0].decode('utf-8')
		c.privmsg("You said: " + text)

	def do_command(self, nick, msg):
		c = self.connection
        
		if "help" in msg:
			c.notice(nick, "I will meet you at " + self.known_users[nick])
			c.notice(nick, "Let me know if its done our if should forget you ...")

		elif "done" in msg:
			channelOut=self.pii.getLog(self.known_users[nick])
			channelPath = channelOut[:-3]

			#Convert IRC-Log to SPL-Source
			saveMerica(channelPath, nick)

			#Compile SPL-Source to C
			args = ["/usr/bin/python", "./Spl/splc.py", channelPath + nick+".spl"]
			fout = open (channelPath  + nick + ".c", 'w')
			process = subprocess.run(args, stdout=fout, stderr=subprocess.PIPE )
			fout.close()

			if process.stderr:
				c.notice(nick, "You know what grinds my wheels?")
				c.notice(nick, "   " + process.stderr.decode().replace("\r", "").replace("\n", " "))
				return
			else:
				c.notice(nick, "I hear your word ...")

			#Compile C
			args = ["/usr/bin/gcc", "-lm", "-I", ".", channelPath + nick + ".c", "-o", channelPath + nick + ".out"]
			process = subprocess.run(args, stderr=subprocess.PIPE)

			if not process.stderr:
				args = [channelPath + nick+ ".out"]
				try:
					process = subprocess.run(args, stdout=subprocess.PIPE)
				except:
					c.notice(nick, "What a mess ...")
			else:
				return
			c.notice(nick, "Your logic is fine")


			#Bash it
			try:
				args = ["/usr/bin/bash", "-c", process.stdout.decode()]
				process = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, timeout=5, cwd="./flagDir")
				if process.stderr:
					c.notice(nick, "You know grinds my wheels?")
					c.notice(nick, "   " + process.stderr.decode().replace("\r", "").replace("\n", " "))
				else:
					c.notice(nick, "Can you feel the joy, pumping thru your venes?")
					c.notice(nick, process.stdout.decode().replace("\r", "").replace("\n", " "))

			except subprocess.TimeoutExpired:
				c.notice(nick, "I cannot wait much longer ...")
			except:
				c.notice(nick, "No thats wrong ...")
			
			print("... done\n")

		elif "forget" in msg:
			args = ["rm", "-rf", self.pii.getLog(self.known_users[nick])]
			self.pii.leaveChannel(self.known_users[nick])
			del(self.known_users[nick])
			c.notice(nick, "Do I know you, fellow pale?")
			subprocess.run(args)
		else:
			c.notice(nick, "Forgive me! But I heard you declaim:")
			c.notice(nick, "Reading. I'm sure from some Greek tragedy?")



def main():
	global debugOut 
	debugOut = True
	server = "localhost" #"irc.hackint.org"
	port = 6642
	channel = "#testchannel" 
	nickname = "shakedbot" 
	 
	bot = TestBot(channel, nickname, server, port)
	bot.start()

if __name__ == "__main__":
	main()
